/*

This program will import data from a CSV file to a database.

1. Enter the name of the CSV file and the database.  This program opens databases
   with the DBFNTX driver.

2. Click on MAP to map the fields
   a. Drag from the left browse (CSV) to the right browse (DBF) when mapping fields
   b. Drag from two or more CSV fields to the DBF to concatenate the data
   c. Right click the right browse for formatting options.
   d. Click on Save MAP to save the mapped fields
   e. Click on Restore MAP to restore mapped fields from previously saved .MAP file

3. Save the MAP to a *.MAP file to restore the map info later when importing another
   CSV file with the same structure.

*/

#INCLUDE "dcdialog.CH"
#INCLUDE "dcprint.CH"
#INCLUDE "appevent.CH"

EXTERNAL DC_CapFirst

STATIC soDropDialog, scFieldName, snFieldNum

FUNCTION Main()

LOCAL GetList[0], GetOptions, cCSVFile := Pad('',100), ;
      cDBFFile := Pad('',100), oBrowse1, oBrowse2, aCSV[0], aDBF[0], ;
      oToolBar, aRef[6], oProgress, lImporting := .f., cDelim := ',', ;
      lTabDelimited := .t., aDir

aDir := Directory('*.Map')
IF Len(aDir) > 0
  cCSVFile := Pad(Strtran(Upper(aDir[1,1]),'.MAP','.CSV'),100)
  cDBFFile := Pad(Strtran(Upper(aDir[1,1]),'.MAP','.DBF'),100)
ENDIF

@ 0,0 DCSAY 'CSV File to Import From:' SAYSIZE 0 SAYBOTTOM RESIZE DCGUI_RESIZE_REPOSONLY_Y
@ 1,0 DCGET cCSVFile GETSIZE 25 POPUP {|c|DC_PopFile(c)} RESIZE DCGUI_RESIZE_REPOSONLY_Y

@ 0,32 DCSAY 'DBF File to Import To:' SAYSIZE 0 SAYBOTTOM RESIZE DCGUI_RESIZE_REPOSONLY_Y
@ 1,32 DCGET cDBFFile GETSIZE 25 POPUP {|c|DC_PopFile(c)} RESIZE DCGUI_RESIZE_REPOSONLY_Y

@ DCGUI_ROW, DCGUI_COL + 20 DCSAY 'CSV Delimiter' GET cDelim SAYSIZE 0 SAYBOTTOM ;
      GETFONT '16.Lucida Console Bold' WHEN {||!lTabDelimited} GETID 'DELIM'

@ DCGUI_ROW, DCGUI_COL + 20 DCCHECKBOX lTabDelimited PROMPT 'TAB Delimeted' ;
      ACTION {||IIF(lTabDelimited,cDelim := Chr(9),cDelim := ','), ;
                DC_GetRefresh(GetList,'DELIM')}

@ 3,0 DCBROWSE oBrowse1 DATA aCSV SIZE 30, 20 ;
      PRESENTATION DC_BrowPres() FONT '9.Lucida Console' ;
      NOHSCROLL ;
      NOSOFTTRACK ;
      POINTER aRef[5] ;
      RESIZE DCGUI_RESIZE_RESIZEONLY_Y

DCBROWSECOL ELEMENT 1 HEADER 'CSV Field Name' WIDTH 40 ;
      PARENT oBrowse1

@ 3,32 DCBROWSE oBrowse2 DATA aDBF SIZE 60, 20 ;
      PRESENTATION DC_BrowPres() FONT '9.Lucida Console' ;
      NOHSCROLL ;
      NOSOFTTRACK ;
      RBSELECT ;
      FIT ;
      POINTER aRef[6] ;
      RESIZE DCGUI_RESIZE_RESIZEONLY_Y ;

DCBROWSECOL ELEMENT 1 HEADER 'DBF Field Name' WIDTH 18 ;
      PARENT oBrowse2

DCBROWSECOL ELEMENT 12 HEADER 'Default Value' WIDTH 25 PARENT oBrowse2

DCBROWSECOL DATA {||Format(oBrowse2:dataSource,oBrowse2:arrayElement,1)} ;
      HEADER 'CSV Field #1' WIDTH 25 ;
      PARENT oBrowse2

DCBROWSECOL DATA {||Format(oBrowse2:dataSource,oBrowse2:arrayElement,2)} ;
      HEADER 'CSV Field #2' WIDTH 25 ;
      PARENT oBrowse2

DCBROWSECOL DATA {||Format(oBrowse2:dataSource,oBrowse2:arrayElement,3)} ;
      HEADER 'CSV Field #3' WIDTH 25 ;
      PARENT oBrowse2

 @ 23.5,0 DCPROGRESS oProgress ;
        TYPE XBPSTATIC_TYPE_TEXT ;
        PERCENT ;
        PERCENTCOLOR GRA_CLR_RED ;
        FONT '10.Arial Bold' ;
        EVERY 1 ;
        COLOR GRA_CLR_CYAN, GRA_CLR_WHITE ;
        SIZE 80,1 ;
        RADIUS 20 ;
        OUTLINE ;
        DYNAMIC

@ 25, 0 DCTOOLBAR oToolBar SIZE 130, 1.2 ;
        BUTTONSIZE 13,1.2

DCADDBUTTON CAPTION '~Map Fields' PARENT oToolBar ;
   ACCELKEY xbeK_ALT_M ;
   TOOLTIP 'Open Files and Create Field Map Browses' ;
   WHEN {||!Empty(cCSVFile) .AND. !Empty(cDBFFile) .AND. !lImporting} ;
   ACTION {||MapFields(cCSVFile,cDBFFile,oBrowse1,oBrowse2,cDelim), ;
             DC_GetRefresh(GetList)}

DCADDBUTTON CAPTION 'Create Database' PARENT oToolBar ;
   TOOLTIP 'Create an empty database from the field names in the CSV file' ;
   WHEN {||!Empty(cCSVFile) .AND. !Empty(cDBFFile) .AND. !lImporting} ;
   ACTION {||CreateDatabase(cCSVFile,cDBFFile,cDelim), ;
             DC_GetRefresh(GetList)}

DCADDBUTTON CAPTION 'Import Data' PARENT oToolBar ;
   TOOLTIP 'Start the Import of Mapped Fields' ;
   WHEN {||!Empty(cCSVFile) .AND. !Empty(cDBFFile) .AND. !lImporting .AND. !Empty(aDBF) .AND. !Empty(aCSV)} ;
   ACTION {||ImportData(cCSVFile,cDBFFile,oBrowse1,oBrowse2,oProgress,@lImporting,GetList,cDelim)}

DCADDBUTTON CAPTION 'Stop Import' PARENT oToolBar ;
   TOOLTIP 'Cancel Import' ;
   WHEN {||lImporting} ;
   ACTION {||lImporting := .F., ;
             DC_GetRefresh(GetList)}

DCADDBUTTON CAPTION 'Save Map' PARENT oToolBar ;
   TOOLTIP 'Save Field Map' ;
   WHEN {||!Empty(aDBF)} ;
   ACTION {||SaveFieldMap(oBrowse2, cDBFFile)}

DCADDBUTTON CAPTION 'Restore Map' PARENT oToolBar ;
   TOOLTIP 'Restore Field Map' ;
   WHEN {||!Empty(aDBF)} ;
   ACTION {||RestoreFieldMap(oBrowse2,cDBFFile)}

DCADDBUTTON CAPTION 'Print Map' PARENT oToolBar ;
   TOOLTIP 'Print Field Map' ;
   WHEN {||!Empty(aDBF)} ;
   ACTION {||PrintFieldMap(oBrowse2,aDBF)}

DCADDBUTTON CAPTION 'Block Ops' PARENT oToolBar ;
   TOOLTIP 'Block Operations' ;
   ACTION {||BlockOperations(@cCSVFile,@cDbfFile,oBrowse1,oBrowse2,cDelim,GetList,oProgress)}

DCGETOPTIONS ;
   SAYWIDTH 120 ;
   SAYRIGHTBOTTOM ;
   RESIZE ;
   TABSTOP

DCREAD GUI ;
   FIT ;
   TITLE 'Import CSV File to DBF' ;
   OPTIONS GetOptions ;
   SETAPPWINDOW ;
   EVAL {||aRef[3] := oBrowse1, ;
           aRef[4] := oBrowse2} ;
   HANDLER dragHandler REFERENCE aRef

RETURN Nil

* -------------

STATIC FUNCTION Format( aDBF, nElement, nColumn )

LOCAL cString := '', cFunction

IF Empty(nElement)
  RETURN ''
ELSEIF nElement > Len(aDBF)
  RETURN ''
ELSEIF nColumn == 1
  cString := aDBF[nElement,3]
  cFunction := aDBF[nElement,5]
  IF !Empty(cFunction)
    cString := Strtran(cFunction,'<x>',cString)
  ENDIF
ELSEIF nColumn == 2
  cString := aDBF[nElement,6]
  cFunction := aDBF[nElement,8]
  IF !Empty(cFunction)
    cString := Strtran(cFunction,'<x>',cString)
  ENDIF
ELSEIF nColumn == 3
  cString := aDBF[nElement,9]
  cFunction := aDBF[nElement,11]
  IF !Empty(cFunction)
    cString := Strtran(cFunction,'<x>',cString)
  ENDIF
ENDIF

RETURN cString

* -------------

STATIC FUNCTION SaveFieldMap( oBrowse2, cDbfFile )

LOCAL aDBF, cArrayFile := Alltrim(Strtran(Upper(cDbfFile),'.DBF','.MAP'))

aDBF := oBrowse2:dataSource

DC_ASave( aDBF, cArrayFile )

DCMSGBOX 'Field Map saved to: ' + cArrayFile

RETURN nil

* -------------

STATIC FUNCTION RestoreFieldMap( oBrowse, cDbfFile )

LOCAL aDBF, i, cArrayFile := Alltrim(Strtran(Upper(cDbfFile),'.DBF','.MAP'))

aDBF := DC_ARestore( cArrayFile )

IF Valtype(aDBF) == 'A'
  FOR i := 1 TO Len(aDBF)
    ASize(aDBF[i],12)
    DEFAULT aDBF[i,12] := ''
  NEXT
  oBrowse:dataSource := aDBF
  oBrowse:refreshAll()
  DCMSGBOX 'Field Map restored from: ' + cArrayFile
ELSE
  DC_WinAlert( 'No field map found: ' + cArrayFile )
ENDIF

RETURN nil

* -------------

STATIC FUNCTION PrintFieldMap( oBrowse )

LOCAL i, oPrinter, nRow := 0, lAllFields := .f., GetList[0], lStatus, aDBF

aDBF := oBrowse:dataSource

@ 0,0 DCRADIO lAllFields VALUE .t. PROMPT 'Print All Fields'
@ 1,0 DCRADIO lAllFields VALUE .f. PROMPT 'Print Only Fields with Mappings'

DCREAD GUI FIT TITLE 'Print Map' MODAL TO lStatus ADDBUTTONS
IF !lStatus
  RETURN .f.
ENDIF

DCPRINT ON TO oPrinter ORIENTATION 2
IF Valtype(oPrinter) # 'O' .OR. !oPrinter:lActive
  RETURN .f.
ENDIF

DCPRINT SIZE 50,132
FOR i := 1 TO Len(aDBF)

  IF nRow == 0
    DCPRINT FONT '12.Lucida Console Bold' PRINTER oPrinter
    nRow := 1
    @ nRow, 5 DCPRINT SAY 'DBF Field Name' PRINTER oPrinter
    @ nRow, 25 DCPRINT SAY 'Default Value' PRINTER oPrinter
    @ nRow, 50 DCPRINT SAY 'CSV Field #1' PRINTER oPrinter
    @ nRow, 75 DCPRINT SAY 'CSV Field #2' PRINTER oPrinter
    @ nRow, 100 DCPRINT SAY 'CSV Field #3' PRINTER oPrinter
    DCPRINT FONT '12.Lucida Console' PRINTER oPrinter
    nRow += 2
  ENDIF

  IF !lAllFields .AND. Empty(aDBF[i,12]) .AND. Empty(aDBF[i,3]) .AND. Empty(aDBF[i,6]) .AND. Empty(aDBF[i,9])
    LOOP
  ENDIF

  @ nRow, 5 DCPRINT SAY aDBF[i,1]  PRINTER oPrinter
  @ nRow, 25 DCPRINT SAY aDBF[i,12] PRINTER oPrinter
  @ nRow, 50 DCPRINT SAY aDBF[i,3] PRINTER oPrinter
  @ nRow, 75 DCPRINT SAY aDBF[i,6] PRINTER oPrinter
  @ nRow, 100 DCPRINT SAY aDBF[i,9] PRINTER oPrinter

  nRow++
  IF nRow > 50
    nRow := 0
    DCPRINT EJECT PRINTER oPrinter
  ENDIF

NEXT

DCPRINT OFF PRINTER oPrinter

RETURN .t.

* -------------

STATIC FUNCTION DragHandler( nEvent, mp1, mp2, oXbp, oDlg, aGetList, aRef )

LOCAL aCSV := aRef[3]:dataSource, aDBF := aRef[4]:dataSource, ;
      oBrowse1 := aRef[3], oBrowse2 := aRef[4], nPointer, nOffset, ;
      aPos, oMenu, GetList[0], nColumn, oColumn, nCol

IF Valtype(oXbp)=='O' .AND. oXbp:isDerivedFrom('DC_XbpGet')

  RETURN DCGUI_NONE

ELSEIF nEvent == xbeM_LbDown .AND. oXbp:isDerivedFrom('XbpCellGroup') .AND. ;
   oXbp:setParent():parent == oBrowse1

  DC_CompleteEvents()

  nPointer := aRef[5]
  IF nPointer > 0 .AND. Len(aCSV) > 0
    scFieldName := aCSV[nPointer,1]
    snFieldNum := nPointer
    soDropDialog := DropDialog( scFieldName )
  ENDIF

ELSEIF nEvent == xbeM_LbUp

  IF Valtype(soDropDialog) == 'O'
    soDropDialog:destroy()
    soDropDialog := nil
  ENDIF

  IF oXbp:isDerivedFrom('XbpCellGroup') .AND. oXbp:setParent():parent == oBrowse2 .AND. ;
    !Empty(scFieldName)
    nColumn := 0
    oColumn := oXbp:setParent()
    FOR nCol := 1 TO oBrowse2:colCount
      IF oColumn == oBrowse2:getColumn(nCol)
        nColumn := nCol
        EXIT
      ENDIF
    NEXT
    nOffset := oXbp:CellFromPos(mp1) - oBrowse2:RowPos  // calculate moused cell position offset from current record
    nPointer := oBrowse2:arrayElement + nOffset
    IF nColumn == 3
      aDBF[nPointer,3] := scFieldName
      aDBF[nPointer,4] := snFieldNum
    ELSEIF nColumn == 4
      aDBF[nPointer,6] := scFieldName
      aDBF[nPointer,7] := snFieldNum
    ELSEIF nColumn == 5
      aDBF[nPointer,9] := scFieldName
      aDBF[nPointer,10] := snFieldNum
    ENDIF
    oBrowse2:forceStable()
    oBrowse2:refreshAll()
  ENDIF
  scFieldName := ''
  snFieldNum := 0

ELSEIF nEvent == xbeM_Motion

  IF Valtype(soDropDialog) = 'O'
    aPos := DC_CalcAbsolutePosition(mp1,oXbp)
    aPos[2] -= (soDropDialog:currentSize()[2] + 30)
    soDropDialog:setPos( aPos )
  ENDIF

ELSEIF nEvent == xbeM_RbDown .AND. oXbp:isDerivedFrom('XbpCellGroup') ;
      .AND. oXbp:setParent():parent == oBrowse2

  DC_CompleteEvents()

  nColumn := 0
  oColumn := oXbp:setParent()
  FOR nCol := 1 TO oBrowse2:colCount
    IF oColumn == oBrowse2:getColumn(nCol)
      nColumn := nCol
      EXIT
    ENDIF
  NEXT

  DCSUBMENU oMenu PROMPT 'Menu'
    DCMENUITEM 'Delete' PARENT oMenu ;
      ACTION {||DeleteItem( oXbp, aDBF, oBrowse2, oColumn, nColumn, mp1 )}
    DCMENUITEM SEPARATOR PARENT oMenu
    DCMENUITEM 'Function(x)' PARENT oMenu ;
      ACTION {||FieldFunction( oXbp, aDBF, oBrowse2, oColumn, nColumn, mp1 )}
    DCMENUITEM SEPARATOR PARENT oMenu
    DCMENUITEM 'Default Value' PARENT oMenu ;
      ACTION {||DefaultValue( oXbp, aDBF, oBrowse2, oColumn, nColumn, mp1 )}

    DCREAD GUI EXIT PARENT AppDeskTop()
    oMenu:popup( oXbp, mp1, 2, XBPMENU_PU_DEFAULT + XBPMENU_PU_MOUSE_RBDOWN )

ENDIF

RETURN DCGUI_NONE

* ------------

STATIC FUNCTION DeleteItem( oXbp, aDBF, oBrowse2, oColumn, nColumn, mp1 )

LOCAL nOffset, nPointer

nOffset := oXbp:CellFromPos(mp1) - oBrowse2:RowPos
nPointer := oBrowse2:arrayElement + nOffset
IF nColumn == 3
  aDBF[nPointer,3] := ''
  aDBF[nPointer,4] := 0
ELSEIF nColumn == 4
  aDBF[nPointer,6] := ''
  aDBF[nPointer,7] := 0
ELSEIF nColumn == 5
  aDBF[nPointer,9] := ''
  aDBF[nPointer,10] := 0
ENDIF

oBrowse2:refreshCurrent()

RETURN nil

* ------------

STATIC FUNCTION FieldFunction( oXbp, aDBF, oBrowse2, oColumn, nColumn, mp1 )

LOCAL nOffset, nPointer

nOffset := oXbp:CellFromPos(mp1) - oBrowse2:RowPos
nPointer := oBrowse2:arrayElement + nOffset
IF nColumn == 3
  PickFunction( aDBF[nPointer], 5 )
ELSEIF nColumn == 4
  PickFunction( aDBF[nPointer], 8 )
ELSEIF nColumn == 5
  PickFunction( aDBF[nPointer], 11 )
ENDIF

oBrowse2:refreshCurrent()

RETURN nil

* ------------

STATIC FUNCTION DefaultValue( oXbp, aDBF, oBrowse2, oColumn, nColumn, mp1 )

LOCAL nOffset, nPointer

nOffset := oXbp:CellFromPos(mp1) - oBrowse2:RowPos
nPointer := oBrowse2:arrayElement + nOffset
SetDefaultValue( aDBF[nPointer], 12 )

oBrowse2:refreshCurrent()

RETURN nil

* ------------

STATIC FUNCTION SetDefaultValue( aField, nElement )

LOCAL GetList[0], lStatus

LOCAL cValue := Space(100)

@ 0,0 DCSAY 'Default Value' GET cValue SAYSIZE 0 SAYBOTTOM GETSIZE 50

DCREAD GUI FIT ADDBUTTONS MODAL TITLE 'Enter Default' ENTEREXIT TO lStatus

IF lStatus
  cValue := Trim(cValue)
  aField[nElement] := cValue
ENDIF

RETURN nil

* ------------

STATIC FUNCTION PickFunction( aField, nElement )

LOCAL cFieldName, cFunction, GetList[0], GetOptions, aFuncts, ;
      nPointer := 1, oBrowse, lStatus

aFuncts := { ;
   { 'None',                                 '<x>'                   }, ;
   { 'Left Pad with Zeros',                  'FieldPadZero(<x>,<y>)' }, ;
   { 'Upper Case',                           'Upper(<x>)'            }, ;
   { 'Lower Case',                           'Lower(<x>)'            }, ;
   { 'Capitalize First Letter of Each Word', 'DC_CapFirst(<x>)'      }, ;
   { 'Left Pad with Spaces',                 'FieldPadL(<x>,<y>)'    }, ;
   { 'Convert CSV Date to Xbase Date',       'ConvertCSVDate(<x>)'   }  ;
   }

@ 0,0 DCBROWSE oBrowse DATA aFuncts SIZE 10,10 FIT ;
      PRESENTATION DC_BrowPres() ;
      ITEMSELECTED {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)} ;
      POINTER nPointer ;
      FONT '8.Lucida Console' ;
      CURSORMODE XBPBRW_CURSOR_ROW

DCBROWSECOL ELEMENT 1 HEADER 'Description' WIDTH 40 PARENT oBrowse
DCBROWSECOL ELEMENT 2 HEADER 'Syntax' WIDTH 20 PARENT oBrowse

DCGETOPTIONS RESIZE RESIZEDEFAULT DCGUI_RESIZE_RESIZEONLY

DCREAD GUI ;
   FIT ;
   ADDBUTTONS ;
   MODAL ;
   TO lStatus ;
   TITLE 'Choose a function' ;
   OPTIONS GetOptions

IF lStatus
  IF nPointer = 1
    aField[nElement] := ''
  ELSE
    aField[nElement] := aFuncts[nPointer,2]
  ENDIF
ENDIF

RETURN nil


* ------------

STATIC FUNCTION DropDialog( cText )

LOCAL GetList[0], GetOptions, oDialog

@ 0, 0 DCSAY cText ;
      COLOR GRA_CLR_BLACK, XBPSYSCLR_TRANSPARENT ;
      FONT '10.Courier' SAYSIZE 0

DCGETOPTIONS NOTITLEBAR ORIGIN XBPDLG_ORIGIN_MOUSE ;
      FITPAD 0 COLOR { 240, 240, 255 } ;
      BORDER XBPDLG_NO_BORDER

DCREAD GUI FIT OPTIONS GetOptions PARENT @oDialog EXIT

RETURN oDialog

* -------------

STATIC FUNCTION MapFields( cCSVFile, cDBFFile, oBrowse1, oBrowse2, cDelim )

LOCAL nHandle, cLine, aTokens, bError, i, aStru, aDBF, aCSV, nFound

aDBF := oBrowse2:dataSource
aCSV := oBrowse1:dataSource
cCSVFile := Alltrim(cCSVFile)
cDBFFile := Alltrim(cDBFFile)
nHandle := DC_TxtOpen(cCSVFile)

IF nHandle > 0
  cLine := DC_TxtLine( nHandle )
  aTokens := DC_TokenArray( cLine, cDelim,.t. )
  ASize(aCSV,0)
  FOR i := 1 TO Len(aTokens)
    AAdd( aCSV, { Strtran(aTokens[i],'"','') } )
  NEXT
  DC_TxtClose(nHandle)
ELSE
  DC_Winalert( 'Cannot open file: ' + cCSVFile )
  RETURN .F.
ENDIF

dbCloseAll()
BEGIN SEQUENCE
bError := ErrorBlock( {||DbfError(cDBFfile)} )
USE (cDbfFile) VIA 'DBFNTX' NEW ALIAS 'DBF' EXCLUSIVE
aStru := DBF->(dbStruct())
ASize(aDBF,0)
FOR i := 1 TO Len(aStru)
  AAdd( aDBF, { aStru[i,1], aStru[i,2], '', 0, '','', 0, '','', 0,'','' } )
  nFound := AScan( aCSV, {|a|a[1] == aStru[i,1]} )
  IF nFound > 0
    ATail(aDBF)[3] := aStru[i,1]
    ATail(aDBF)[4] := i
  ENDIF
NEXT

END SEQUENCE
ErrorBlock( bError )

RETURN .t.

* -------------

STATIC FUNCTION DbfError( cDBFFile )

DC_WinAlert( 'Error opening DBF: ' + cDBFFile )
BREAK

RETURN nil

* -------------

STATIC FUNCTION ImportData( cCSVFile, cDBFFile, oBrowse1, oBrowse2, oProgress, ;
                            lImporting, GetList, cDelim )

LOCAL nHandle, aTokens, cLine, i, nMaxCount := 0, cDbfField, cCsvField, ;
      nCsvNum, xData, cDbfType, nCount, aCSV, aDBF, cFunct, cDefault

PRIVATE cData

aCSV := oBrowse1:dataSource
aDBF := oBrowse2:dataSource

nHandle := DC_TxtOpen(cCSVFile)

IF nHandle > 0
  lImporting := .T.
  DC_GetRefresh(GetList)
  DC_TxtSkip(nHandle,1)
  nCount := 0
  DO WHILE !DC_TxtEof(nHandle)
    nMaxCount++
    DC_TxtSkip(nHandle,1)
  ENDDO
  DC_TxtGoTo(nHandle,2)

  DO WHILE !DC_TxtEof(nHandle) .AND. lImporting
    Sleep(1)
    cLine := DC_TxtLine( nHandle )
    aTokens := DC_TokenArray( cLine, cDelim,.t. )
    DBF->(dbAppend())
    FOR i := 1 TO Len(aDBF)
      cDbfField := 'DBF->' + aDbf[i,1]
      cDbfType := aDbf[i,2]
      cCsvField := aDbf[i,3]
      nCsvNum := aDbf[i,4]
      cDefault := aDbf[i,12]
      IF !Empty(cDefault)
        IF cDbfType $ 'CM'
          xData := cDefault
        ELSEIF cDbfType = 'N'
          xData := Val(cDefault)
        ELSEIF cDbfType = 'D'
          xData := CtoD(cDefault)
        ELSEIF cDbfType = 'L'
          xData := Upper(Left(cDefault,1)) $ 'YT'
        ENDIF
        &(cDbfField) := xData
      ENDIF
      IF nCsvNum > 0
        IF nCsvNum > Len(aTokens)
          LOOP
        ENDIF
        cFunct := aDbf[i,5]
        M->cData := aTokens[nCsvNum]
        IF !Empty(cFunct)
          cFunct := Strtran(cFunct,'<x>','M->cData')
          cFunct := Strtran(cFunct,'<y>',cDbfField)
          xData := &(cFunct)
        ELSEIF cDbfType $ 'CM'
          xData := M->cData
        ELSEIF cDbfType = 'N'
          xData := Val(M->cData)
        ELSEIF cDbfType = 'D'
          xData := CtoD(M->cData)
        ELSEIF cDbfType = 'L'
          xData := Upper(Left(M->cData,1)) $ 'YT'
        ENDIF
        &(cDbfField) := xData
      ENDIF
      nCsvNum := aDbf[i,7]
      IF nCsvNum > 0
        cFunct := aDbf[i,8]
        M->cData := aTokens[nCsvNum]
        IF !Empty(cFunct)
          cFunct := Strtran(cFunct,'<x>','M->cData')
          cFunct := Strtran(cFunct,'<y>',cDbfField)
          xData := &(cFunct)
          &(cDbfField) := Trim(&(cDbfField)) + ' ' + xData
        ELSEIF cDbfType $ 'CM'
          xData := M->cData
          &(cDbfField) := Trim(&(cDbfField)) + ' ' + xData
        ELSEIF cDbfType = 'N'
          xData := M->cData
          &(cDbfField) := &(cDbfField) + xData
        ENDIF
      ENDIF
      nCsvNum := aDbf[i,10]
      IF nCsvNum > 0
        cFunct := aDbf[i,11]
        M->cData := aTokens[nCsvNum]
        IF !Empty(cFunct)
          cFunct := Strtran(cFunct,'<x>','M->cData')
          cFunct := Strtran(cFunct,'<y>',cDbfField)
          xData := &(cFunct)
          &(cDbfField) := Trim(&(cDbfField)) + ' ' + xData
        ELSEIF cDbfType $ 'CM'
          xData := M->cData
          &(cDbfField) := Trim(&(cDbfField)) + ' ' + xData
        ELSEIF cDbfType = 'N'
          xData := M->cData
          &(cDbfField) := &(cDbfField) + xData
        ENDIF
      ENDIF
    NEXT
    DC_GetProgress( oProgress, ++nCount, nMaxCount )
    DC_CompleteEvents()
    DC_TxtSkip(nHandle,1)
  ENDDO
  DC_TxtClose(nHandle)
ENDIF

lImporting := .F.
DC_GetRefresh(GetList)

RETURN nil

* -------------

FUNCTION ConvertCSVDate( cDate )

cDate := Substr( cDate, 5) + Substr( cDate, 1, 4 )

RETURN StoD( cDate )

* -------------

PROC appsys
RETURN

* -------------

FUNCTION FieldPadZero( cSourceData, cDestData )

cSourceData := PadL( cSourceData, Len(cDestData) )

RETURN DC_PadZero(cSourceData)

* -------------

FUNCTION FieldPadSpaces( cSourceData, cDestData )

cSourceData := PadL( cSourceData, Len(cDestData) )

RETURN cSourceData

* -------------

FUNCTION CreateDatabase( cCsvFile, cDBFFile, cDelim, lPrompt )

LOCAL nHandle, i, cLine, aTokens, aStru[0], cDatabase, lStatus := .t., ;
      aMap[0], cMapFile

DEFAULT lPrompt := .t.

cCsvFile := Alltrim(cCsvFile)
cDbfFile := Alltrim(cDbfFile)
cMapFile := Substr(cDbfFile,1,At('.',cDbfFile)-1) + '.MAP'

nHandle := DC_TxtOpen(cCsvFile)
cLine := Alltrim(DC_TxtLine(nHandle))

aTokens := DC_TokenArray(cLine,cDelim)

FOR i := 1 TO Len(aTokens)
  aTokens[i] := Alltrim(Strtran(aTokens[i],'"',''))
  IF Empty(aTokens[i])
    LOOP
  ENDIF
  IF i == 1
    AAdd(aStru,{aTokens[i],'C',10,0})
  ELSE
    AAdd(aStru,{aTokens[i],'N',12,0})
  ENDIF
  AAdd( aMap, { ATail(aStru)[1], ATail(aStru)[2],ATail(aStru)[1],i,'','',0,'','',0,'',''} )
NEXT

DC_TxtClose(nHandle)

IF File(cDBFFile)
   DCMSGBOX cDBFFile + ' exists!  Overwrite?' YESNO TO lStatus
ENDIF

IF lStatus
  dbCreate( cDBFFile, aStru )
  IF lPrompt
    DC_ASave( aMap, cMapFile)
    DCMSGBOX 'Done!' TIMEOUT 3
  ENDIF
ENDIF

RETURN lStatus

* ------------

STATIC FUNCTION BlockOperations( cCSVFile, cDbfFile, oBrowse1, oBrowse2, cDelim, aMainGetList, oProgress )

LOCAL GetList[0], GetOptions, cTxtDir, cDestDir, oStatus

cTxtDir := Space(100)
cDestDir := Space(100)

@ 0,0 DCSAY 'Text file Directory:' SAYSIZE 0
@ 1,1 DCGET cTxtDir POPUP {|c|DC_PopDir(c)} GETSIZE 80 ;
      VALID {||TestDir(cTxtDir)}

@ 3,0 DCSAY 'Destination Database file Directory:' SAYSIZE 0
@ 4,1 DCGET cDestDir POPUP {|c|DC_PopDir(c)} GETSIZE 80 ;
      VALID {||TestDir(cDestDir)}

@ 6,0 DCSAY 'Status:' SAYSIZE 0
@ 7,1 DCSAY '' SAYSIZE 100 FONT '10.Lucida Console' SAYLEFTBOTTOM OBJECT oStatus COLOR GRA_CLR_BLUE, GRA_CLR_WHITE

@ 10,0 DCPUSHBUTTON CAPTION 'Start Import' SIZE 12 ;
   ACTION {||BlockImport(cTxtDir,cDestDir,@cCsvFile,@cDbfFile,oStatus,oBrowse1,oBrowse2,cDelim, ;
                         aMainGetList,oProgress,GetList)}

DCGETOPTIONS TABSTOP NORESIZE

DCREAD GUI FIT TITLE 'Block Operations' OPTIONS GetOptions MODAL SETAPPWINDOW

RETURN nil

* -----------

STATIC FUNCTION TestDir( cDir )

LOCAL lStatus

cDir := Alltrim(cDir)

lStatus := DC_IsDir(cDir)

IF !lStatus
  DC_Winalert( cDir + ' does not exist' )
ENDIF

RETURN lStatus

* -----------

STATIC FUNCTION BlockImport( cTxtDir, cDestDir, cCsvFile, cDbfFile, oStatus, oBrowse1, oBrowse2, ;
                             cDelim, aMainGetList, oProgress, aGetList )

LOCAL aTxtDir, i, lImporting := .t., cFirstField, cNtxFile

aTxtDir := Directory( Alltrim(cTxtDir) + '\*.txt' )

cDelim := Chr(9)

FOR i := 1 TO Len(aTxtDir)

  cCsvFile := Upper(Alltrim(cTxtDir) + '\' + aTxtDir[i,1])
  cDbfFile := Alltrim(cDestDir) + '\' + DC_Path(Strtran(cCsvFile,'.TXT','.DBF'),.t.)
  oStatus:setCaption( 'Creating Database: ' + cDbfFile )
  IF CreateDatabase( cCsvFile, cDbfFile, cDelim, .f. )
    Sleep(100)
    oStatus:setCaption( 'Mapping: ' + cCsvFile )
    MapFields( cCsvFile, cDbfFile, oBrowse1, oBrowse2, cDelim )
    Sleep(100)
    oStatus:setCaption( 'Importing to Database: ' + cDbfFile )

    cCsvFile := Pad(cCsvFile,100)
    cDbfFile := Pad(cDbfFile,100)
    DC_GetRefresh(aMainGetList)

    ImportData( cCsvFile, cDbfFile, oBrowse1, oBrowse2, oProgress, ;
                @lImporting, aMainGetList, cDelim )
    Sleep(100)

    cNtxFile := Strtran(cDbfFile,'.DBF','.NTX')
    oStatus:setCaption( 'Creating Index File: ' + cDbfFile )

    cFirstField := DbStruct()[1,1]
    INDEX ON FIELD->IDRSSD TO (cNtxFile)

  ENDIF

NEXT

dbCloseAll()
oStatus:setCaption('Done!')
Sleep(100)
DC_ReadGuiEvent(DCGUI_EXIT_OK,aGetList)

RETURN nil
